/****************************************************************************
 * Copyright (c) 2010-2012, EBM WebSourcing - All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
package com.ebmwebsourcing.easycommons.xml;

import javax.xml.stream.XMLOutputFactory;

import com.ebmwebsourcing.easycommons.pooling.GenericResourcePool;
import com.ebmwebsourcing.easycommons.pooling.PoolException;
import com.ebmwebsourcing.easycommons.pooling.PoolPolicy;
import com.ebmwebsourcing.easycommons.pooling.ResourceHandler;

/**
 * This class represent a {@link XMLOutputFactory} pool, which provided {@link XMLOutputFactory}
 * object as resource.
 * 
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class XMLOutputFactoryResourcePool {

    /**
     * {@link XMLOutputFactory} resource handler to manage {@link XMLOutputFactory} life cycle methods
     */
    private static class XMLOutputFactoryResourceHandler implements ResourceHandler<XMLOutputFactory> {

        public XMLOutputFactoryResourceHandler() {
        }

        @Override
        public XMLOutputFactory create() {
            return XMLOutputFactory.newInstance();
        }

        @Override
        public void onRelease(XMLOutputFactory xmlOutputFactory) {
        }

        @Override
        public void onTake(XMLOutputFactory xmlOutputFactory) {            
        }
    }
    
    private final GenericResourcePool<XMLOutputFactory> xmlOutputFactoryPool;

    /**
     * Allowing to instantiate a new {@link XMLOutputFactoryResourcePool} containing {@link XMLOutputFactory} resources.
     * 
     * @param minPoolSize
     *            The minimum number of {@link XMLOutputFactory} instances in the pool (created at the
     *            initialization).
     * @param maxPoolSize
     *            the maximum number of {@link XMLOutputFactory} instances in the current pool (limit of the
     *            pool). It must be greater or equals to the specified minSize.
     *            The maximum value is Integer.MAX_VALUE
     * @param poolPolicy
     *            the {@link PoolPolicy} to adopt when the maximum size is reached. it
     *            cannot be null.
     */
    public XMLOutputFactoryResourcePool(int minPoolSize, int maxPoolSize,
            PoolPolicy poolPolicy) {
        XMLOutputFactoryResourceHandler xmlOutputFactoryResourceHandler = new XMLOutputFactoryResourceHandler();
        this.xmlOutputFactoryPool = new GenericResourcePool<XMLOutputFactory>(
                xmlOutputFactoryResourceHandler, minPoolSize, maxPoolSize, poolPolicy);
    }

    /**
     * Take one unused {@link XMLOutputFactory} in the current pool. After getting a
     * {@link XMLOutputFactory} from the pool and before returning a
     * {@link XMLOutputFactory}, the method onTake() of the {@link XMLOutputFactory}
     * resource handler is called.
     * 
     * @return one {@link XMLOutputFactory}
     * 
     * @throws PoolException
     *             if the current thread is interrupted for the pool policy WAIT
     *             or if there is no more available resource in the pool for the
     *             pool policy REJECT
     * 
     */
    public XMLOutputFactory take() {
        return this.xmlOutputFactoryPool.take();
    }

    /**
     * Release the specified {@link XMLOutputFactory} After putting back the
     * {@link XMLOutputFactory} in the pool, the method onRelease() of the
     * {@link XMLOutputFactory} resource handler is called.
     * 
     * @param xmlOutputFactory
     *            The {@link XMLOutputFactory} to release
     */
    public final void release(final XMLOutputFactory xmlOutputFactory) {
        this.xmlOutputFactoryPool.release(xmlOutputFactory);
    }
}
